﻿using StbLoadingApp.Helpers;
using StbLoadingApp.Model;
using StbLoadingApp.Services;
using StbLoadingApp.Services.NavigationService;
using Syncfusion.ListView.XForms;
using System;
using System.Collections.ObjectModel;
using System.Linq;
using System.Windows.Input;
using Xamarin.Forms;

namespace StbLoadingApp.ViewModel
{
    public class TripPageViewModel : ViewModelBase
    {
        #region Properties
        private bool isLoading;
        public bool IsLoading
        {
            get { return isLoading; }
            set
            {
                isLoading = value;
                OnPropertyChanged("IsLoading");
            }
        }
        private DateTime _dateFrom;
        public DateTime DateFrom
        {
            get { return _dateFrom; }
            set
            {
                if (_dateFrom != value || value != null)
                {
                    _dateFrom = value;
                    OnPropertyChanged();
                }
            }
        }

        private SearchBar searchBar = null;
        private Command<object> filterCommand;
        public Command<object> FilterCommand
        {
            get { return filterCommand; }
            protected set { filterCommand = value; }
        }
        private Command<SfListView> tripLoadedCommand;
        public Command<SfListView> TripLoadedCommand
        {
            get { return tripLoadedCommand; }
            protected set { tripLoadedCommand = value; }
        }
        private Command<SfListView> tripChangedCommand;
        public Command<SfListView> TripChangedCommand
        {
            get { return tripChangedCommand; }
            protected set { tripChangedCommand = value; }
        }


        private INavigationService _navigationService;
        private ITripService _tripService;
        private IMasterDataService _masterDataService;
        public ICommand PreferencesCommand { get; private set; }
        public ICommand SearchCommand { get; private set; }
        public ICommand AssignCommand { get; private set; }
        private ObservableCollection<Trip> _tripList;
        public ObservableCollection<Trip> TripList
        {
            get { return _tripList; }
            set
            {
                _tripList = value;
                OnPropertyChanged();
            }
        }
        #endregion
        public TripPageViewModel(INavigationService navigationService,
                                 ITripService tripService,
                                 IMasterDataService masterDataService)
        {
            IsLoading = true;

            _navigationService = navigationService;
            _tripService = tripService;
            _masterDataService = masterDataService;

            SearchCommand = new Command(Search);
            AssignCommand = new Command(AssignTo);
            PreferencesCommand = new Command(Preferences);

            filterCommand = new Command<object>(OnTextChanged);
            tripChangedCommand = new Command<SfListView>(TripChanged);
            tripLoadedCommand = new Command<SfListView>(TripLoaded);
        }
        public override async void OnAppearing(object navigationContext)
        {
            DateFrom = await _masterDataService.GetWorkDayAsync(0);

            if (_tripList == null)
            {
                var result = await _tripService.GetTripsAsync(DateFrom, 1);
                TripList = new ObservableCollection<Trip>(result.OrderBy(x => x.Truck));
            }
            base.OnAppearing(navigationContext);
            IsLoading = false;
        }
        public override void OnDisappearing()
        {
            base.OnDisappearing();
        }
        # region private members
        private void AssignTo(object obj)
        {
//            Trip t = obj as Trip;
            _navigationService.NavigateTo<EmployeePageViewModel>(obj);
        }
        private async void Search()
        {
            IsLoading = true;
            TripList = await _tripService.GetTripsAsync(DateFrom, 1);
            IsLoading = false;
        }
        private void Preferences()
        {
            _navigationService.NavigateTo<ServicesPageViewModel>(null);
        }
        #endregion
        #region SfListView events
        private void TripLoaded(SfListView listView)
        {
            //            await Application.Current.MainPage.DisplayAlert("Alert", "Listview Loaded", "OK", "Cancel");
        }
        private void TripChanged(SfListView listView)
        {
            var selectedItem = listView.SelectedItems[0];
            Trip s = selectedItem as Trip;
        }
        #endregion
        #region SearchBar events
        private void OnTextChanged(object obj)
        {
            var grid = (obj as Grid);
            searchBar = grid.Children[0] as SearchBar;
            var listview = grid.Children[1] as SfListView;
            if (listview.DataSource != null)
            {
                listview.DataSource.Filter = FilterTrips;
                listview.DataSource.RefreshFilter();
            }
            listview.RefreshView();
        }
        private bool FilterTrips(object obj)
        {
            if (searchBar == null || searchBar.Text == null)
                return true;

            var taskInfo = obj as Trip;
            return (taskInfo.Driver.ToLower().Contains(searchBar.Text.ToLower())
                || taskInfo.Truck.ToLower().Contains(searchBar.Text.ToLower()));
        }
        #endregion
    }
}
